# funary

![npm version](https://img.shields.io/npm/v/funary.svg) [![Build Status](https://travis-ci.org/EESimulations/funary.svg)](https://travis-ci.org/EESimulations/funary)

<img src="https://i.imgur.com/xedwvyA.png" width="200" height="200">

Arbitrary precision binary operations library for Javascript.

# [NPM](https://www.npmjs.com/package/funary) Install

```
npm install funary
```

## Gates

The library has XOR,AND,OR,NAND,XNOR, and NOR gates.

```javascript
var funary = require('funary'); 
funary.gates.and([1,1,1,1,1,0]) //0
funary.gates.xor([0,0,0,0,1,0]) //1
funary.gates.xor([1,0,0,0,0,1]) //0
funary.gates.or([1,0,0,0,0,1]) //1
```

## Bitwise operations

Similar selection to gates. AND,OR , ... etc.

```javascript
funary.and([0,1,0,1,0,1,0,1,1,1],[1,0,0,0,1,1,1,1,0,1]) //[0,0,0,0,0,1,0,1,0,1]
//   0101010111
//&  1000111101
//-------------
//   0000010101
funary.or([0,1,0,1,0,1,0,1,1,1],[1,0,0,0,1,1,1,1,0,1]) //[1,1,0,1,1,1,1,1,1,1]
//   0101010111
//|  1000111101
//-------------
//   1101111111
```

## Big endian operations

```javascript
var funary = require('funary');

//Adding binary (size depends on largest array)
funary.big.NADDER([0,1,0,1,0,1,0,1,1,1],[1,0,0,0,1,1,1,1,0,1]) //[0,1,1,1,0,0,1,0,1,0,0]
//  0101010111
//+ 1000111101
//------------
// 01110010100

//Multiplication
funary.big.NMULTIPLY([1,0,1,0,1,1,0,0,1],[1,1,1,0,0,1,1,1]) //[1,0,0,1,1,0,1,1,1,0,1,0,0,1,1,1,1]
//          101011001
//X          11100111
//-------------------
//  10011011101001111

//Decimal to binary
funary.big.toUnsignedBinary(12) //[1, 1, 0, 0]
//Binary to decimal
funary.big.toUnsignedDecimal([1, 1, 0, 1]) //13
```

## Little endian operations

```javascript
var funary = require('funary');

//Adding binary (size depends on largest array)
funary.little.NADDER([0,1,0,1,0,1,0,1,1,1],[1,0,0,0,1,1,1,1,0,1]) //[1,1,0,1,1,0,0,1,0,1,1]
//  0101010111
//+ 1000111101
//-------------
//  11011001011

//Multiplication
funary.little.NMULTIPLY([1,1,1,0,1,0,1,0,1],[1,0,1,1]) //[1,1,0,1,0,1,1,0,1,0,0,0,1]
//  111010101
//X 1011
//---------------
//  1101011010001

//Decimal to binary
funary.little.toUnsignedBinary(12) //[0, 0, 1, 1]
//Binary to decimal
funary.little.toUnsignedDecimal([1, 1, 1, 0, 1]) //23
```

## Bit shifts

```javascript
var funary = require('funary'); 
funary.left([0,0,1,0,1,1],2) //[1,0,1,1,0,0]
funary.right([0,0,1,0,1,1],2) //[0,0,0,0,1,0]
```

## Other operations

```javascript
var funary = require('funary'); 
funary.not(1) //0
funary.not(0) //1
funary.invert([0,1,1,0,0,1,1,1]) //[1,0,0,1,1,0,0,0]
//Truncation
funary.righttrunc([1, 1, 1, 0, 1, 0, 1, 0],4) //[1, 1, 1, 0]
funary.lefttrunc([1, 1, 1, 0, 1, 0, 1, 0],5) //[0, 1, 0, 1, 0]
//Utils
funary.arrayToString([1,1,0,1,0,1,1,1,1,0,0]) //"11010111100"
funary.stringToArray('001000001111') //[0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1]
//Combinations
funary.getCombinations(3) //[[1,1,1],[0,1,1],[1,0,1],[0,0,1],[1,1,0],[0,1,0],[1,0,0],[0,0,0]]
//Zeros
funary.zeros(4) //[0,0,0,0]
```

## Related projects

[DLCSim](http://eesimulations.github.io/DLCSim)

Digital Logic Circuit Simulation library. Create and simulate high level integrated circuits. Depends on funary.


